/*
 * Copyright 2002-2004 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.appfuse.richclient.ui;

import org.appfuse.model.User;
import org.appfuse.service.UserExistsException;
import org.appfuse.service.UserManager;
import org.springframework.richclient.application.event.LifecycleApplicationEvent;
import org.springframework.richclient.command.ActionCommandExecutor;
import org.springframework.richclient.form.CompoundForm;
import org.springframework.richclient.wizard.AbstractWizard;
import org.springframework.richclient.wizard.FormBackedWizardPage;
import org.springframework.richclient.wizard.WizardDialog;
import org.springframework.util.Assert;

public class NewOwnerWizard extends AbstractWizard implements ActionCommandExecutor {
    private WizardDialog wizardDialog;

    private CompoundForm wizardForm;

//    private Clinic clinic;
    private UserManager userManager;

    public NewOwnerWizard() {
        super("newOwnerWizard");
    }

    public void setUserManager(UserManager userManager) {
        Assert.notNull(userManager, "The clinic property is required");
        this.userManager = userManager;
    }

    public void addPages() {
        addPage(new FormBackedWizardPage(new OwnerGeneralForm(wizardForm.getFormModel())));
        addPage(new FormBackedWizardPage(new OwnerAddressForm(wizardForm.getFormModel())));
    }

    public void execute() {
        if (wizardDialog == null) {
            wizardDialog = new WizardDialog(this);
            wizardForm = new CompoundForm();
        }
        wizardForm.setFormObject(new User());
        wizardDialog.showDialog();
    }

    protected boolean onFinish() {
        User newUser = (User)getNewUser();
        try {
        userManager.saveUser(newUser);
        } catch (UserExistsException e){
            logger.debug(e);
        }
        getApplicationContext()
                .publishEvent(new LifecycleApplicationEvent(LifecycleApplicationEvent.CREATED, newUser));
        return true;
    }

    private User getNewUser() {
        wizardForm.commit();
        return (User)wizardForm.getFormObject();
    }

}