/*
 * Copyright 2002-2004 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.appfuse.richclient.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;

import org.appfuse.service.UserExistsException;
import org.appfuse.service.UserManager;
import org.springframework.binding.form.NestingFormModel;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.core.closure.support.Block;
import org.springframework.remoting.RemoteAccessException;
import org.springframework.remoting.RemoteConnectFailureException;
import org.springframework.richclient.application.PageComponentContext;
import org.springframework.richclient.application.event.LifecycleApplicationEvent;
import org.springframework.richclient.application.support.AbstractView;
import org.springframework.richclient.command.ActionCommand;
import org.springframework.richclient.command.CommandGroup;
import org.springframework.richclient.command.support.AbstractActionCommandExecutor;
import org.springframework.richclient.command.support.GlobalCommandIds;
import org.springframework.richclient.dialog.CompositeDialogPage;
import org.springframework.richclient.dialog.ConfirmationDialog;
import org.springframework.richclient.dialog.FormBackedDialogPage;
import org.springframework.richclient.dialog.InputApplicationDialog;
import org.springframework.richclient.dialog.TabbedDialogPage;
import org.springframework.richclient.dialog.TitledPageApplicationDialog;
import org.springframework.richclient.dialog.TreeCompositeDialogPage;
import org.springframework.richclient.form.FormModelHelper;
import org.springframework.richclient.preference.PreferenceStore;
import org.springframework.richclient.progress.TreeStatusBarUpdater;
import org.appfuse.model.Role;
import org.appfuse.model.User;
import org.appfuse.richclient.ui.preference.CompositeDialogPageType;
import org.appfuse.richclient.ui.preference.PetClinicAppearance;
import org.springframework.richclient.tree.FocusableTreeCellRenderer;
import org.springframework.richclient.util.PopupMenuMouseListener;
import org.springframework.util.Assert;

public class OwnerManagerView extends AbstractView implements ApplicationListener {

    private UserManager userManager;

    private String ownerLastName = "";

    private JTree ownersTree;

    private DefaultTreeModel ownersTreeModel;

    private RenameExecutor renameExecutor = new RenameExecutor();

    private DeleteExecutor deleteExecutor = new DeleteExecutor();

    private PropertiesExecutor propertiesExecutor = new PropertiesExecutor();

    private RenameOwnerExecutor renameOwnerExecutor = new RenameOwnerExecutor();

    private RenamePetExecutor renamePetExecutor = new RenamePetExecutor();

    private OwnerPropertiesExecutor ownerPropertiesExecutor = new OwnerPropertiesExecutor();

    private PetPropertiesExecutor petPropertiesExecutor = new PetPropertiesExecutor();

    private NewPetAction newPetCommand = new NewPetAction();

    public void setUserManager(UserManager userManager) {
        Assert.notNull(userManager, "The userManager property is required");
        this.userManager = userManager;
    }

    public void setLastNameToDisplay(String ownerLastName) {
        this.ownerLastName = ownerLastName;
    }

    protected void registerLocalCommandExecutors(PageComponentContext context) {
        context.register("renameCommand", renameExecutor);
        context.register(GlobalCommandIds.DELETE, deleteExecutor);
        context.register(GlobalCommandIds.PROPERTIES, propertiesExecutor);
    }

    protected JComponent createControl() {
        JPanel view = new JPanel(new BorderLayout());
        createOwnerManagerTree();
        JScrollPane sp = new JScrollPane(ownersTree);
        view.add(sp, BorderLayout.CENTER);
        return view;
    }

    private void createOwnerManagerTree() {
        
        DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Users");
        try {
            Collection users = userManager.getUsers(null);
        } catch (RemoteAccessException e) {
            System.out.println("Working offline:"+e);
        }
//        for (Iterator i = owners.iterator(); i.hasNext();) {
//            User user = (User)i.next();
//            DefaultMutableTreeNode ownerNode = new DefaultMutableTreeNode(user);
//            for (Iterator j = user.getRoleList().iterator(); j.hasNext();) {
//                ownerNode.add(new DefaultMutableTreeNode(j.next()));
//            }
//            rootNode.add(ownerNode);
//        }
        DefaultMutableTreeNode ownerNode = new DefaultMutableTreeNode("User 1");
        ownerNode.add(new DefaultMutableTreeNode("admin"));
        ownerNode.add(new DefaultMutableTreeNode("superAdmin"));
        rootNode.add(ownerNode);
        ownerNode = new DefaultMutableTreeNode("User 2");
        ownerNode.add(new DefaultMutableTreeNode("admin"));
        rootNode.add(ownerNode);
        
        
        this.ownersTreeModel = new DefaultTreeModel(rootNode);
        this.ownersTree = new JTree(ownersTreeModel);
        ownersTree.setShowsRootHandles(false);
//        ownersTree.addTreeSelectionListener(new TreeStatusBarUpdater(getStatusBar()) {
//            public String getSelectedObjectName() {
//                User selectedOwner = getSelectedUser();
//                if (selectedOwner != null) {
//                    return selectedOwner.getFirstName() + " " + selectedOwner.getLastName();
//                }
//                else {
//                    return "Owners";
//                }
//            }
//        });
//        ownersTree.addTreeSelectionListener(new TreeSelectionListener() {
//            public void valueChanged(TreeSelectionEvent e) {
//                updateCommands();
//            }
//        });
//        ownersTree.addMouseListener(new PopupMenuMouseListener() {
//            protected boolean onAboutToShow(MouseEvent e) {
//                return !isRootOrNothingSelected();
//            }
//
//            protected JPopupMenu getPopupMenu() {
//                return getSelectedUser() != null ? createOwnerPopupContextMenu() : createPetPopupContextMenu();
//            }
//        });
//        ownersTree.addMouseListener(new MouseAdapter() {
//            public void mouseClicked(MouseEvent e) {
//                if (e.getClickCount() == 2 && e.getButton() == MouseEvent.BUTTON1 && propertiesExecutor.isEnabled()) {
//                    propertiesExecutor.execute();
//                }
//            }
//        });
        ownersTree.setCellRenderer(getTreeCellRenderer());
        ownersTree.setRootVisible(true);
    }

    private User getSelectedUser() {
        DefaultMutableTreeNode node = getSelectedOwnerNode();
        if (node != null) {
            return (User)node.getUserObject();
        }
        else {
            return null;
        }
    }

    private DefaultMutableTreeNode getSelectedOwnerNode() {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)ownersTree.getLastSelectedPathComponent();
        if (node == null || !(node.getUserObject() instanceof User)) {
            return null;
        }
        else {
            return node;
        }
    }

    private Role getSelectedRole() {
        DefaultMutableTreeNode node = getSelectedPetNode();
        if (node != null) {
            return (Role)node.getUserObject();
        }
        else {
            return null;
        }
    }

    private DefaultMutableTreeNode getSelectedPetNode() {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)ownersTree.getLastSelectedPathComponent();
        if (node == null || !(node.getUserObject() instanceof Role)) {
            return null;
        }
        else {
            return node;
        }
    }

    private void updateCommands() {
        int treeSelectionCount = ownersTree.getSelectionCount();
        if (isRootOrNothingSelected()) {
            renameExecutor.setEnabled(false);
            deleteExecutor.setEnabled(false);
            propertiesExecutor.setEnabled(false);
        }
        else if (treeSelectionCount == 1) {
            renameExecutor.setEnabled(true);
            deleteExecutor.setEnabled(true);
            propertiesExecutor.setEnabled(true);
        }
        else if (treeSelectionCount > 1) {
            renameExecutor.setEnabled(false);
            deleteExecutor.setEnabled(true);
            propertiesExecutor.setEnabled(false);
        }
        newPetCommand.setEnabled(getSelectedUser() != null);
    }

    private boolean isRootOrNothingSelected() {
        return ownersTree.getSelectionCount() == 0
                || (ownersTree.getSelectionCount() == 1 && ownersTree.isRowSelected(0));
    }

    private DefaultTreeCellRenderer treeCellRenderer = new FocusableTreeCellRenderer() {

        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded,
                boolean leaf, int row, boolean hasFocus) {
            super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
            if (node.isRoot()) {
                this.setIcon(getIconSource().getIcon("folder.icon"));
            }
            else {
                Object userObject = node.getUserObject();
                if (userObject instanceof User) {
                    User o = (User)userObject;
                    this.setText(o.getFirstName() + " " + o.getLastName());
                    this.setIcon(getIconSource().getIcon("owner.bullet"));
                }
                else if (userObject instanceof Role) {
                    Role p = (Role)userObject;
                    this.setText("<html>" + p.getName() + " <i>( aaa )");
                    this.setIcon(getIconSource().getIcon("pet.bullet"));
                }
            }
            return this;
        }
    };

    public TreeCellRenderer getTreeCellRenderer() {
        return treeCellRenderer;
    }

    private JPopupMenu createOwnerPopupContextMenu() {
        // rename, separator, delete, addPet separator, properties
        CommandGroup group = getWindowCommandManager().createCommandGroup(
                "ownerViewTableOwnerCommandGroup",
                new Object[] {"renameCommand", "separator", "deleteCommand", "separator", newPetCommand, "separator",
                        "propertiesCommand"});
        return group.createPopupMenu();
    }

    private JPopupMenu createPetPopupContextMenu() {
        // rename, separator, delete, separator, properties
        CommandGroup group = getWindowCommandManager().createCommandGroup("ownerViewTablePetCommandGroup",
                new Object[] {"renameCommand", "separator", "deleteCommand", "separator", "propertiesCommand"});
        return group.createPopupMenu();
    }

    public void onApplicationEvent(ApplicationEvent e) {
        if (e instanceof LifecycleApplicationEvent) {
            LifecycleApplicationEvent le = (LifecycleApplicationEvent)e;
            if (le.getEventType() == LifecycleApplicationEvent.CREATED && le.objectIs(User.class)) {
                if (ownersTree != null) {
                    DefaultMutableTreeNode root = (DefaultMutableTreeNode)ownersTreeModel.getRoot();
                    root.add(new DefaultMutableTreeNode(le.getObject()));
                    ownersTreeModel.nodeStructureChanged(root);
                }
            }
        }
    }

    private class RenameExecutor extends AbstractActionCommandExecutor {
        public void execute() {
            if (getSelectedUser() != null) {
                renameOwnerExecutor.execute();
            }
            else {
                renamePetExecutor.execute();
            }
        }
    }

    private class PropertiesExecutor extends AbstractActionCommandExecutor {
        public void execute() {
            if (getSelectedUser() != null) {
                ownerPropertiesExecutor.execute();
            }
            else {
                petPropertiesExecutor.execute();
            }
        }
    }

    private class DeleteExecutor extends AbstractActionCommandExecutor {
        public void execute() {
            int ownerCount = 0;
            int petCount = 0;
            final List nodesToDelete = new ArrayList();	
            TreePath[] paths = ownersTree.getSelectionPaths();
            for (int i = 0; i < paths.length; i++) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)paths[i].getLastPathComponent();
                if (node.isRoot()) {
                    continue;
                }
                if (node.getUserObject() instanceof User) {
                    ownerCount++;
                }
                else {
                    petCount++;
                }
                nodesToDelete.add(node);
            }

            ConfirmationDialog dialog = new ConfirmationDialog() {
                protected void onConfirm() {
                    for (Iterator i = nodesToDelete.iterator(); i.hasNext();) {
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode)i.next();
                        ownersTreeModel.removeNodeFromParent(node);
                        if (node.getUserObject() instanceof User) {
                            // clinic.deleteOwner((Owner) node.getUserObject());
                        }
                        else {
                            // clinic.deletePet((Pet) node.getUserObject());
                        }
                    }
                }
            };
            if (ownerCount > 0 && petCount > 0) {
                dialog.setTitle(getMessage("confirmDeleteOwnerAndPetDialog.title"));
                dialog.setConfirmationMessage(getMessage("confirmDeleteOwnerAndPetDialog.label"));
            }
            else if (ownerCount > 0) {
                dialog.setTitle(getMessage("confirmDeleteOwnerDialog.title"));
                dialog.setConfirmationMessage(getMessage("confirmDeleteOwnerDialog.label"));
            }
            else {
                dialog.setTitle(getMessage("confirmDeletePetDialog.title"));
                dialog.setConfirmationMessage(getMessage("confirmDeletePetDialog.label"));
            }

            dialog.showDialog();
        }
    }

    private class RenameOwnerExecutor extends AbstractActionCommandExecutor {
        public void execute() {
            final User user = getSelectedUser();
            InputApplicationDialog renameDialog = new InputApplicationDialog(user, "firstName");
            renameDialog.setTitle(getMessage("renameOwnerDialog.title"));
            renameDialog.setInputLabelMessage("renameOwnerDialog.label");
            renameDialog.setParent(getWindowControl());
            renameDialog.setFinishAction(new Block() {

                public void handle(Object o) {
                    try {
                        userManager.saveUser(user);
                    } catch (UserExistsException e){
                        logger.error(e);
                    }
                    getSelectedOwnerNode().setUserObject(user);
                    ownersTreeModel.nodeChanged(getSelectedOwnerNode());
                }
            });
            renameDialog.showDialog();
        }
    }

    private class RenamePetExecutor extends AbstractActionCommandExecutor {
        public void execute() {
            final Role pet = getSelectedRole();
            InputApplicationDialog renameDialog = new InputApplicationDialog(pet, "name");
            renameDialog.setTitle(getMessage("renamePetDialog.title"));
            renameDialog.setInputLabelMessage("renamePetDialog.label");
            renameDialog.setParent(getWindowControl());
            renameDialog.setFinishAction(new Block() {

                public void handle(Object o) {
//                    userManager.storePet(pet);
//                    getSelectedPetNode().setUserObject(pet);
//                    ownersTreeModel.nodeChanged(getSelectedPetNode());
                }
            });
            renameDialog.showDialog();
        }
    }

    private class OwnerPropertiesExecutor extends AbstractActionCommandExecutor {

        private NestingFormModel ownerFormModel;

        private OwnerGeneralForm ownerGeneralForm;

        private CompositeDialogPage compositePage;

        public void execute() {
            final User owner = getSelectedUser();
            ownerFormModel = FormModelHelper.createCompoundFormModel(owner);
            ownerGeneralForm = new OwnerGeneralForm(ownerFormModel);

            PreferenceStore ps = (PreferenceStore)getApplicationContext().getBean("preferenceStore");
            CompositeDialogPageType type = (CompositeDialogPageType)ps.getLabeledEnum(PetClinicAppearance.DIALOG_PAGE_TYPE);
            if (type == null || type.equals(CompositeDialogPageType.TABBED)) {
                compositePage = new TabbedDialogPage("ownerProperties");
            }
            else {
                compositePage = new TreeCompositeDialogPage("ownerProperties");
            }
            compositePage.addForm(ownerGeneralForm);
            compositePage.addForm(new OwnerAddressForm(ownerFormModel));

            TitledPageApplicationDialog dialog = new TitledPageApplicationDialog(compositePage, getWindowControl()) {
                protected void onAboutToShow() {
                    ownerGeneralForm.requestFocusInWindow();
                    setEnabled(compositePage.isPageComplete());
                }

                protected boolean onFinish() {
                    ownerFormModel.commit();
                    try {
                        userManager.saveUser(owner);
                    } catch (UserExistsException e){
                        logger.error(e);
                    }
                    ownersTreeModel.nodeChanged(getSelectedOwnerNode());
                    return true;
                }
            };
            dialog.showDialog();
        }
    }

    private class PetPropertiesExecutor extends AbstractActionCommandExecutor {

        public void execute() {
            final Role role = getSelectedRole();
            final PetForm petForm = new PetForm(FormModelHelper.createFormModel(role), false);
            final FormBackedDialogPage dialogPage = new FormBackedDialogPage(petForm);

            TitledPageApplicationDialog dialog = new TitledPageApplicationDialog(dialogPage, getWindowControl()) {
                protected void onAboutToShow() {
                    petForm.requestFocusInWindow();
                    setEnabled(dialogPage.isPageComplete());
                }

                protected boolean onFinish() {
//                    petForm.commit();
//                    roleManager.(role);
//                    ownersTreeModel.nodeChanged(getSelectedPetNode());
                    return true;
                }
            };
            dialog.showDialog();
        }
    }

    private class NewPetAction extends ActionCommand {

        public NewPetAction() {
            super("newPetCommand");
        }

        protected void doExecuteCommand() {
//            final Role newPet = new JdbcPet();
//            final PetForm petForm = new PetForm(FormModelHelper.createFormModel(newPet), true);
//            final FormBackedDialogPage dialogPage = new FormBackedDialogPage(petForm);

//            TitledPageApplicationDialog dialog = new TitledPageApplicationDialog(dialogPage, getWindowControl()) {
//                protected void onAboutToShow() {
//                    petForm.requestFocusInWindow();
//                    setEnabled(dialogPage.isPageComplete());
//                }
//
//                protected boolean onFinish() {
//                    petForm.commit();
//                    getSelectedUser().addPet(newPet);
//                    userManager.storePet(newPet);
//                    DefaultMutableTreeNode ownerNode = getSelectedOwnerNode();
//                    ownerNode.add(new DefaultMutableTreeNode(newPet));
//                    ownersTreeModel.nodeStructureChanged(ownerNode);
//                    return true;
//                }
//            };
//            dialog.showDialog();

        }
    }

}